@@ -38,8 +38,9 @@ class Role(Enum):
 
 class LLM:
 
-    def __init__(self, model="gpt-3.5-turbo-1106", use_assistants_api=True):
+    def __init__(self, model="gpt-3.5-turbo-1106", use_assistants_api=True, head_number=2):
         self.model = model
+        self.head_number = head_number
         if use_assistants_api:
             self._call_openai_llm = self._get_response_from_assistant
             self.client = OpenAI()
@@ -79,10 +80,10 @@ class LLM:
         Select a prompt between the one saving the plot and the one calculating some math.
         """
         print(f"{BLUE}SELECTING A PROMPT:{RESET}")
-        temlate_for_plot_planner = Prompts.generate_steps_for_plot_show.format(input=user_query, df_head=df.head(1))
+        temlate_for_plot_planner = Prompts.generate_steps_for_plot_show.format(input=user_query, df_head=df.head(self.head_number))
         if save_plot_name is not None:
-            temlate_for_plot_planner = Prompts.generate_steps_for_plot_save.format(input=user_query, plotname=save_plot_name, df_head=df.head(1))
-        temlate_for_math_planner = Prompts.generate_steps_no_plot.format(df_head=df.head(1), input=user_query)
+            temlate_for_plot_planner = Prompts.generate_steps_for_plot_save.format(input=user_query, plotname=save_plot_name, df_head=df.head(self.head_number))
+        temlate_for_math_planner = Prompts.generate_steps_no_plot.format(df_head=df.head(self.head_number), input=user_query)
 
         prompt_branch = RunnableBranch(
             (lambda x: x["topic"] == "plot", PromptTemplate.from_template(temlate_for_plot_planner)),
@@ -160,10 +161,10 @@ class LLM:
     def plan_steps_with_gpt(self, user_query, df, save_plot_name):
         query_type = self.tag_query_type(user_query, df, save_plot_name)
 
-        temlate_for_plot_planner = Prompts.generate_steps_for_plot_show.format(input=user_query, df_head=df.head(1))
+        temlate_for_plot_planner = Prompts.generate_steps_for_plot_show.format(input=user_query, df_head=df.head(self.head_number))
         if save_plot_name is not None:
-            temlate_for_plot_planner = Prompts.generate_steps_for_plot_save.format(input=user_query, plotname=save_plot_name, df_head=df.head(1))
-        temlate_for_math_planner = Prompts.generate_steps_no_plot.format(df_head=df.head(1), input=user_query)
+            temlate_for_plot_planner = Prompts.generate_steps_for_plot_save.format(input=user_query, plotname=save_plot_name, df_head=df.head(self.head_number))
+        temlate_for_math_planner = Prompts.generate_steps_no_plot.format(df_head=df.head(self.head_number), input=user_query)
 
         selected_prompt = temlate_for_math_planner
         if query_type == "plot":
@@ -173,10 +174,12 @@ class LLM:
 
         return self._call_openai_llm(selected_prompt, role=Role.PLANNER), (selected_prompt, query_type)
 
-    def generate_code_with_gpt(self, user_query, df, plan):
-        prompt = Prompts.generate_code.format(input=user_query, df_head=df.head(1), plan=plan)
+    def generate_code_with_gpt(self, user_query, df, plan, show_plot=False, tagged_query_type="general"):
+        prompt = Prompts.generate_code.format(input=user_query, df_head=df.head(self.head_number), plan=plan, head_number=self.head_number)
+        if tagged_query_type == "plot" and not show_plot: # don't include plt.show() in the generated code
+            prompt = Prompts.generate_code_for_plot_save.format(input=user_query, df_head=df.head(self.head_number), plan=plan, head_number=self.head_number)
         return self._call_openai_llm(prompt, role=Role.CODER), prompt
 
-    def fix_generated_code(self, df, code_to_be_fixed, error_message):
-        prompt = Prompts.fix_code.format(code=code_to_be_fixed, df_head=df.head(1), error=error_message)
+    def fix_generated_code(self, df, code_to_be_fixed, error_message, user_query):
+        prompt = Prompts.fix_code.format(code=code_to_be_fixed, df_head=df.head(self.head_number), error=error_message, input=user_query, head_number=self.head_number)
         return self._call_openai_llm(prompt, Role.DEBUGGER), prompt
